# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("p2p.dll")
prototypes = \
    {
        # 
        'PeerFreeData': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pvData"]),
        # 
        'PeerGetItemCount': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEnum", "pCount"]),
        # 
        'PeerGetNextItem': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEnum", "pCount", "pppvItems"]),
        # 
        'PeerEndEnumeration': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEnum"]),
        # 
        'PeerGroupStartup': SimTypeFunction([SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimStruct({"wVersion": SimTypeShort(signed=False, label="UInt16"), "wHighestVersion": SimTypeShort(signed=False, label="UInt16")}, name="PEER_VERSION_DATA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["wVersionRequested", "pVersionData"]),
        # 
        'PeerGroupShutdown': SimTypeFunction([], SimTypeInt(signed=True, label="Int32")),
        # 
        'PeerGroupCreate': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCloud": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzClassifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzGroupPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzCreatorPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ulMemberDataLifetime": SimTypeInt(signed=False, label="UInt32"), "ulPresenceLifetime": SimTypeInt(signed=False, label="UInt32"), "dwAuthenticationSchemes": SimTypeInt(signed=False, label="UInt32"), "pwzGroupPassword": SimTypePointer(SimTypeChar(label="Char"), offset=0), "groupPasswordRole": SimTypeBottom(label="Guid")}, name="PEER_GROUP_PROPERTIES", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pProperties", "phGroup"]),
        # 
        'PeerGroupOpen': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzGroupPeerName", "pwzCloud", "phGroup"]),
        # 
        'PeerGroupJoin': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzInvitation", "pwzCloud", "phGroup"]),
        # 
        'PeerGroupPasswordJoin': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzInvitation", "pwzPassword", "pwzCloud", "phGroup"]),
        # 
        'PeerGroupConnect': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup"]),
        # 
        'PeerGroupConnectByAddress': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "cAddresses", "pAddresses"]),
        # 
        'PeerGroupClose': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup"]),
        # 
        'PeerGroupDelete': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzGroupPeerName"]),
        # 
        'PeerGroupCreateInvitation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Guid"), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzIdentityInfo", "pftExpiration", "cRoles", "pRoles", "ppwzInvitation"]),
        # 
        'PeerGroupCreatePasswordInvitation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "ppwzInvitation"]),
        # 
        'PeerGroupParseInvitation': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCloudName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwScope": SimTypeInt(signed=False, label="UInt32"), "dwCloudFlags": SimTypeInt(signed=False, label="UInt32"), "pwzGroupPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzIssuerPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzSubjectPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzGroupFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzIssuerFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzSubjectFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ftValidityStart": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftValidityEnd": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "cRoles": SimTypeInt(signed=False, label="UInt32"), "pRoles": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "cClassifiers": SimTypeInt(signed=False, label="UInt32"), "ppwzClassifiers": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "pSubjectPublicKey": SimTypePointer(SimTypeBottom(label="CERT_PUBLIC_KEY_INFO"), offset=0), "authScheme": SimTypeInt(signed=False, label="PEER_GROUP_AUTHENTICATION_SCHEME")}, name="PEER_INVITATION_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzInvitation", "ppInvitationInfo"]),
        # 
        'PeerGroupGetStatus': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pdwStatus"]),
        # 
        'PeerGroupGetProperties': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCloud": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzClassifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzGroupPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzCreatorPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ulMemberDataLifetime": SimTypeInt(signed=False, label="UInt32"), "ulPresenceLifetime": SimTypeInt(signed=False, label="UInt32"), "dwAuthenticationSchemes": SimTypeInt(signed=False, label="UInt32"), "pwzGroupPassword": SimTypePointer(SimTypeChar(label="Char"), offset=0), "groupPasswordRole": SimTypeBottom(label="Guid")}, name="PEER_GROUP_PROPERTIES", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "ppProperties"]),
        # 
        'PeerGroupSetProperties': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCloud": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzClassifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzGroupPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzCreatorPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ulMemberDataLifetime": SimTypeInt(signed=False, label="UInt32"), "ulPresenceLifetime": SimTypeInt(signed=False, label="UInt32"), "dwAuthenticationSchemes": SimTypeInt(signed=False, label="UInt32"), "pwzGroupPassword": SimTypePointer(SimTypeChar(label="Char"), offset=0), "groupPasswordRole": SimTypeBottom(label="Guid")}, name="PEER_GROUP_PROPERTIES", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pProperties"]),
        # 
        'PeerGroupEnumMembers': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "dwFlags", "pwzIdentity", "phPeerEnum"]),
        # 
        'PeerGroupOpenDirectConnection': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzIdentity", "pAddress", "pullConnectionId"]),
        # 
        'PeerGroupCloseDirectConnection': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64")], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "ullConnectionId"]),
        # 
        'PeerGroupEnumConnections': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "dwFlags", "phPeerEnum"]),
        # 
        'PeerGroupSendData': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "ullConnectionId", "pType", "cbData", "pvData"]),
        # 
        'PeerGroupRegisterEvent': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"eventType": SimTypeInt(signed=False, label="PEER_GROUP_EVENT_TYPE"), "pType": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="PEER_GROUP_EVENT_REGISTRATION", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "hEvent", "cEventRegistration", "pEventRegistrations", "phPeerEvent"]),
        # 
        'PeerGroupUnregisterEvent': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEvent"]),
        # 
        'PeerGroupGetEventData': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"eventType": SimTypeInt(signed=False, label="PEER_GROUP_EVENT_TYPE"), "Anonymous": SimUnion({"dwStatus": SimTypeInt(signed=False, label="PEER_GROUP_STATUS"), "incomingData": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "ullConnectionId": SimTypeLongLong(signed=False, label="UInt64"), "type": SimTypeBottom(label="Guid"), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_EVENT_INCOMING_DATA", pack=False, align=None), "recordChangeData": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "changeType": SimTypeInt(signed=False, label="PEER_RECORD_CHANGE_TYPE"), "recordId": SimTypeBottom(label="Guid"), "recordType": SimTypeBottom(label="Guid")}, name="PEER_EVENT_RECORD_CHANGE_DATA", pack=False, align=None), "connectionChangeData": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "status": SimTypeInt(signed=False, label="PEER_CONNECTION_STATUS"), "ullConnectionId": SimTypeLongLong(signed=False, label="UInt64"), "ullNodeId": SimTypeLongLong(signed=False, label="UInt64"), "ullNextConnectionId": SimTypeLongLong(signed=False, label="UInt64"), "hrConnectionFailedReason": SimTypeInt(signed=True, label="Int32")}, name="PEER_EVENT_CONNECTION_CHANGE_DATA", pack=False, align=None), "memberChangeData": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "changeType": SimTypeInt(signed=False, label="PEER_MEMBER_CHANGE_TYPE"), "pwzIdentity": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_EVENT_MEMBER_CHANGE_DATA", pack=False, align=None), "hrConnectionFailedReason": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None")}, name="PEER_GROUP_EVENT_DATA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEvent", "ppEventData"]),
        # 
        'PeerGroupGetRecord': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "type": SimTypeBottom(label="Guid"), "id": SimTypeBottom(label="Guid"), "dwVersion": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCreatorId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzModifiedById": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzAttributes": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ftCreation": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftExpiration": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftLastModified": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "securityData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_RECORD", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pRecordId", "ppRecord"]),
        # 
        'PeerGroupAddRecord': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "type": SimTypeBottom(label="Guid"), "id": SimTypeBottom(label="Guid"), "dwVersion": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCreatorId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzModifiedById": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzAttributes": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ftCreation": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftExpiration": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftLastModified": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "securityData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_RECORD", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pRecord", "pRecordId"]),
        # 
        'PeerGroupUpdateRecord': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "type": SimTypeBottom(label="Guid"), "id": SimTypeBottom(label="Guid"), "dwVersion": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzCreatorId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzModifiedById": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzAttributes": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ftCreation": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftExpiration": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftLastModified": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "securityData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_RECORD", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pRecord"]),
        # 
        'PeerGroupDeleteRecord': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pRecordId"]),
        # 
        'PeerGroupEnumRecords': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pRecordType", "phPeerEnum"]),
        # 
        'PeerGroupSearchRecords': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzCriteria", "phPeerEnum"]),
        # 
        'PeerGroupExportDatabase': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzFilePath"]),
        # 
        'PeerGroupImportDatabase': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzFilePath"]),
        # 
        'PeerGroupIssueCredentials': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwzFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pPublicKey": SimTypePointer(SimTypeBottom(label="CERT_PUBLIC_KEY_INFO"), offset=0), "pwzIssuerPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzIssuerFriendlyName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ftValidityStart": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "ftValidityEnd": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "cRoles": SimTypeInt(signed=False, label="UInt32"), "pRoles": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="PEER_CREDENTIAL_INFO", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzSubjectIdentity", "pCredentialInfo", "dwFlags", "ppwzInvitation"]),
        # 
        'PeerGroupExportConfig': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pwzPassword", "ppwzXML"]),
        # 
        'PeerGroupImportConfig': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzXML", "pwzPassword", "fOverwrite", "ppwzIdentity", "ppwzGroup"]),
        # 
        'PeerGroupPeerTimeToUniversalTime': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pftPeerTime", "pftUniversalTime"]),
        # 
        'PeerGroupUniversalTimeToPeerTime': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "pftUniversalTime", "pftPeerTime"]),
        # 
        'PeerGroupResumePasswordAuthentication': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hGroup", "hPeerEventHandle"]),
        # 
        'PeerIdentityCreate': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzClassifier", "pwzFriendlyName", "hCryptProv", "ppwzIdentity"]),
        # 
        'PeerIdentityGetFriendlyName': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "ppwzFriendlyName"]),
        # 
        'PeerIdentitySetFriendlyName': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzFriendlyName"]),
        # 
        'PeerIdentityGetCryptKey': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "phCryptProv"]),
        # 
        'PeerIdentityDelete': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity"]),
        # 
        'PeerEnumIdentities': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phPeerEnum"]),
        # 
        'PeerEnumGroups': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "phPeerEnum"]),
        # 
        'PeerCreatePeerName': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzClassifier", "ppwzPeerName"]),
        # 
        'PeerIdentityGetXML': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "ppwzIdentityXML"]),
        # 
        'PeerIdentityExport': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzIdentity", "pwzPassword", "ppwzExportXML"]),
        # 
        'PeerIdentityImport': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzImportXML", "pwzPassword", "ppwzIdentity"]),
        # 
        'PeerIdentityGetDefault': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppwzPeerName"]),
        # 
        'PeerCollabStartup': SimTypeFunction([SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["wVersionRequested"]),
        # 
        'PeerCollabShutdown': SimTypeFunction([], SimTypeInt(signed=True, label="Int32")),
        # 
        'PeerCollabSignin': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndParent", "dwSigninOptions"]),
        # 
        'PeerCollabSignout': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["dwSigninOptions"]),
        # 
        'PeerCollabGetSigninOptions': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pdwSigninOptions"]),
        # 
        'PeerCollabAsyncInviteContact': SimTypeFunction([SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"applicationId": SimTypeBottom(label="Guid"), "applicationData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_INVITATION", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcContact", "pcEndpoint", "pcInvitation", "hEvent", "phInvitation"]),
        # 
        'PeerCollabGetInvitationResponse': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"action": SimTypeInt(signed=False, label="PEER_INVITATION_RESPONSE_TYPE"), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0), "hrExtendedInfo": SimTypeInt(signed=True, label="Int32")}, name="PEER_INVITATION_RESPONSE", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInvitation", "ppInvitationResponse"]),
        # 
        'PeerCollabCancelInvitation': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInvitation"]),
        # 
        'PeerCollabCloseHandle': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hInvitation"]),
        # 
        'PeerCollabInviteContact': SimTypeFunction([SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"applicationId": SimTypeBottom(label="Guid"), "applicationData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_INVITATION", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimStruct({"action": SimTypeInt(signed=False, label="PEER_INVITATION_RESPONSE_TYPE"), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0), "hrExtendedInfo": SimTypeInt(signed=True, label="Int32")}, name="PEER_INVITATION_RESPONSE", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcContact", "pcEndpoint", "pcInvitation", "ppResponse"]),
        # 
        'PeerCollabAsyncInviteEndpoint': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"applicationId": SimTypeBottom(label="Guid"), "applicationData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_INVITATION", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint", "pcInvitation", "hEvent", "phInvitation"]),
        # 
        'PeerCollabInviteEndpoint': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"applicationId": SimTypeBottom(label="Guid"), "applicationData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_INVITATION", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimStruct({"action": SimTypeInt(signed=False, label="PEER_INVITATION_RESPONSE_TYPE"), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0), "hrExtendedInfo": SimTypeInt(signed=True, label="Int32")}, name="PEER_INVITATION_RESPONSE", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint", "pcInvitation", "ppResponse"]),
        # 
        'PeerCollabGetAppLaunchInfo': SimTypeFunction([SimTypePointer(SimTypePointer(SimStruct({"pContact": SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), "pEndpoint": SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), "pInvitation": SimTypePointer(SimStruct({"applicationId": SimTypeBottom(label="Guid"), "applicationData": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzMessage": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_INVITATION", pack=False, align=None), offset=0)}, name="PEER_APP_LAUNCH_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppLaunchInfo"]),
        # 
        'PeerCollabRegisterApplication': SimTypeFunction([SimTypePointer(SimStruct({"application": SimStruct({"id": SimTypeBottom(label="Guid"), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzDescription": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_APPLICATION", pack=False, align=None), "pwzApplicationToLaunch": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzApplicationArguments": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwPublicationScope": SimTypeInt(signed=False, label="UInt32")}, name="PEER_APPLICATION_REGISTRATION_INFO", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="PEER_APPLICATION_REGISTRATION_TYPE")], SimTypeInt(signed=True, label="Int32"), arg_names=["pcApplication", "registrationType"]),
        # 
        'PeerCollabUnregisterApplication': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="PEER_APPLICATION_REGISTRATION_TYPE")], SimTypeInt(signed=True, label="Int32"), arg_names=["pApplicationId", "registrationType"]),
        # 
        'PeerCollabGetApplicationRegistrationInfo': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="PEER_APPLICATION_REGISTRATION_TYPE"), SimTypePointer(SimTypePointer(SimStruct({"application": SimStruct({"id": SimTypeBottom(label="Guid"), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzDescription": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_APPLICATION", pack=False, align=None), "pwzApplicationToLaunch": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzApplicationArguments": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwPublicationScope": SimTypeInt(signed=False, label="UInt32")}, name="PEER_APPLICATION_REGISTRATION_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pApplicationId", "registrationType", "ppApplication"]),
        # 
        'PeerCollabEnumApplicationRegistrationInfo': SimTypeFunction([SimTypeInt(signed=False, label="PEER_APPLICATION_REGISTRATION_TYPE"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["registrationType", "phPeerEnum"]),
        # 
        'PeerCollabGetPresenceInfo': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimStruct({"status": SimTypeInt(signed=False, label="PEER_PRESENCE_STATUS"), "pwzDescriptiveText": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_PRESENCE_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint", "ppPresenceInfo"]),
        # 
        'PeerCollabEnumApplications': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint", "pApplicationId", "phPeerEnum"]),
        # 
        'PeerCollabEnumObjects': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint", "pObjectId", "phPeerEnum"]),
        # 
        'PeerCollabEnumEndpoints': SimTypeFunction([SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcContact", "phPeerEnum"]),
        # 
        'PeerCollabRefreshEndpointData': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint"]),
        # 
        'PeerCollabDeleteEndpointData': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint"]),
        # 
        'PeerCollabQueryContactData': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint", "ppwzContactData"]),
        # 
        'PeerCollabSubscribeEndpointData': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint"]),
        # 
        'PeerCollabUnsubscribeEndpointData': SimTypeFunction([SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcEndpoint"]),
        # 
        'PeerCollabSetPresenceInfo': SimTypeFunction([SimTypePointer(SimStruct({"status": SimTypeInt(signed=False, label="PEER_PRESENCE_STATUS"), "pwzDescriptiveText": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_PRESENCE_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcPresenceInfo"]),
        # 
        'PeerCollabGetEndpointName': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppwzEndpointName"]),
        # 
        'PeerCollabSetEndpointName': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzEndpointName"]),
        # 
        'PeerCollabSetObject': SimTypeFunction([SimTypePointer(SimStruct({"id": SimTypeBottom(label="Guid"), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "dwPublicationScope": SimTypeInt(signed=False, label="UInt32")}, name="PEER_OBJECT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcObject"]),
        # 
        'PeerCollabDeleteObject': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pObjectId"]),
        # 
        'PeerCollabRegisterEvent': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"eventType": SimTypeInt(signed=False, label="PEER_COLLAB_EVENT_TYPE"), "pInstance": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="PEER_COLLAB_EVENT_REGISTRATION", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hEvent", "cEventRegistration", "pEventRegistrations", "phPeerEvent"]),
        # 
        'PeerCollabGetEventData': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"eventType": SimTypeInt(signed=False, label="PEER_COLLAB_EVENT_TYPE"), "Anonymous": SimUnion({"watchListChangedData": SimStruct({"pContact": SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), "changeType": SimTypeInt(signed=False, label="PEER_CHANGE_TYPE")}, name="PEER_EVENT_WATCHLIST_CHANGED_DATA", pack=False, align=None), "presenceChangedData": SimStruct({"pContact": SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), "pEndpoint": SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), "changeType": SimTypeInt(signed=False, label="PEER_CHANGE_TYPE"), "pPresenceInfo": SimTypePointer(SimStruct({"status": SimTypeInt(signed=False, label="PEER_PRESENCE_STATUS"), "pwzDescriptiveText": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_PRESENCE_INFO", pack=False, align=None), offset=0)}, name="PEER_EVENT_PRESENCE_CHANGED_DATA", pack=False, align=None), "applicationChangedData": SimStruct({"pContact": SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), "pEndpoint": SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), "changeType": SimTypeInt(signed=False, label="PEER_CHANGE_TYPE"), "pApplication": SimTypePointer(SimStruct({"id": SimTypeBottom(label="Guid"), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "pwzDescription": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_APPLICATION", pack=False, align=None), offset=0)}, name="PEER_EVENT_APPLICATION_CHANGED_DATA", pack=False, align=None), "objectChangedData": SimStruct({"pContact": SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), "pEndpoint": SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), "changeType": SimTypeInt(signed=False, label="PEER_CHANGE_TYPE"), "pObject": SimTypePointer(SimStruct({"id": SimTypeBottom(label="Guid"), "data": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None), "dwPublicationScope": SimTypeInt(signed=False, label="UInt32")}, name="PEER_OBJECT", pack=False, align=None), offset=0)}, name="PEER_EVENT_OBJECT_CHANGED_DATA", pack=False, align=None), "endpointChangedData": SimStruct({"pContact": SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), "pEndpoint": SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0)}, name="PEER_EVENT_ENDPOINT_CHANGED_DATA", pack=False, align=None), "peopleNearMeChangedData": SimStruct({"changeType": SimTypeInt(signed=False, label="PEER_CHANGE_TYPE"), "pPeopleNearMe": SimTypePointer(SimStruct({"pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "endpoint": SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), "id": SimTypeBottom(label="Guid")}, name="PEER_PEOPLE_NEAR_ME", pack=False, align=None), offset=0)}, name="PEER_EVENT_PEOPLE_NEAR_ME_CHANGED_DATA", pack=False, align=None), "requestStatusChangedData": SimStruct({"pEndpoint": SimTypePointer(SimStruct({"address": SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "sin6": SimStruct({"sin6_family": SimTypeShort(signed=False, label="UInt16"), "sin6_port": SimTypeShort(signed=False, label="UInt16"), "sin6_flowinfo": SimTypeInt(signed=False, label="UInt32"), "sin6_addr": SimStruct({"u": SimUnion({"Byte": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "Word": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 8)}, name="<anon>", label="None")}, name="IN6_ADDR", pack=False, align=None), "Anonymous": SimUnion({"sin6_scope_id": SimTypeInt(signed=False, label="UInt32"), "sin6_scope_struct": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Value": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="SCOPE_ID", pack=False, align=None)}, name="<anon>", label="None")}, name="SOCKADDR_IN6", pack=False, align=None)}, name="PEER_ADDRESS", pack=False, align=None), "pwzEndpointName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PEER_ENDPOINT", pack=False, align=None), offset=0), "hrChange": SimTypeInt(signed=True, label="Int32")}, name="PEER_EVENT_REQUEST_STATUS_CHANGED_DATA", pack=False, align=None)}, name="<anon>", label="None")}, name="PEER_COLLAB_EVENT_DATA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEvent", "ppEventData"]),
        # 
        'PeerCollabUnregisterEvent': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hPeerEvent"]),
        # 
        'PeerCollabEnumPeopleNearMe': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phPeerEnum"]),
        # 
        'PeerCollabAddContact': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzContactData", "ppContact"]),
        # 
        'PeerCollabDeleteContact': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzPeerName"]),
        # 
        'PeerCollabGetContact': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzPeerName", "ppContact"]),
        # 
        'PeerCollabUpdateContact': SimTypeFunction([SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pContact"]),
        # 
        'PeerCollabEnumContacts': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["phPeerEnum"]),
        # 
        'PeerCollabExportContact': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzPeerName", "ppwzContactData"]),
        # 
        'PeerCollabParseContact': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzNickName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzEmailAddress": SimTypePointer(SimTypeChar(label="Char"), offset=0), "fWatch": SimTypeInt(signed=True, label="Int32"), "WatcherPermissions": SimTypeInt(signed=False, label="PEER_WATCH_PERMISSION"), "credentials": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_CONTACT", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzContactData", "ppContact"]),
        # 
        'PeerNameToPeerHostName': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzPeerName", "ppwzHostName"]),
        # 
        'PeerHostNameToPeerName': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzHostName", "ppwzPeerName"]),
        # 
        'PeerPnrpStartup': SimTypeFunction([SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["wVersionRequested"]),
        # 
        'PeerPnrpShutdown': SimTypeFunction([], SimTypeInt(signed=True, label="Int32")),
        # 
        'PeerPnrpRegister': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"pwzCloudName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzPublishingIdentity": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cAddresses": SimTypeInt(signed=False, label="UInt32"), "ppAddresses": SimTypePointer(SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), offset=0), "wPort": SimTypeShort(signed=False, label="UInt16"), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "payload": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_PNRP_REGISTRATION_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcwzPeerName", "pRegistrationInfo", "phRegistration"]),
        # 
        'PeerPnrpUpdateRegistration': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"pwzCloudName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzPublishingIdentity": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cAddresses": SimTypeInt(signed=False, label="UInt32"), "ppAddresses": SimTypePointer(SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), offset=0), "wPort": SimTypeShort(signed=False, label="UInt16"), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "payload": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_PNRP_REGISTRATION_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegistration", "pRegistrationInfo"]),
        # 
        'PeerPnrpUnregister': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hRegistration"]),
        # 
        'PeerPnrpResolve': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cAddresses": SimTypeInt(signed=False, label="UInt32"), "ppAddresses": SimTypePointer(SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), offset=0), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "payload": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_PNRP_ENDPOINT_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcwzPeerName", "pcwzCloudName", "pcEndpoints", "ppEndpoints"]),
        # 
        'PeerPnrpStartResolve': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcwzPeerName", "pcwzCloudName", "cMaxEndpoints", "hEvent", "phResolve"]),
        # 
        'PeerPnrpGetCloudInfo': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwzCloudName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwScope": SimTypeInt(signed=False, label="PNRP_SCOPE"), "dwScopeId": SimTypeInt(signed=False, label="UInt32")}, name="PEER_PNRP_CLOUD_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pcNumClouds", "ppCloudInfo"]),
        # 
        'PeerPnrpGetEndpoint': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwzPeerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cAddresses": SimTypeInt(signed=False, label="UInt32"), "ppAddresses": SimTypePointer(SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), offset=0), "pwzComment": SimTypePointer(SimTypeChar(label="Char"), offset=0), "payload": SimStruct({"cbData": SimTypeInt(signed=False, label="UInt32"), "pbData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PEER_DATA", pack=False, align=None)}, name="PEER_PNRP_ENDPOINT_INFO", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hResolve", "ppEndpoint"]),
        # 
        'PeerPnrpEndResolve': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hResolve"]),
    }

lib.set_prototypes(prototypes)




from claripy import Annotation


class StackLocationAnnotation(Annotation):
    def __init__(self, offset):
        super(StackLocationAnnotation, self).__init__()

        self.offset = offset

    @property
    def eliminatable(self):
        return False

    @property
    def relocatable(self):
        return False

    def __hash__(self):
        return hash(('stack_location', self.offset))

    def __eq__(self, other):
        if not isinstance(other, StackLocationAnnotation):
            return False

        return self.offset == other.offset


class VariableSourceAnnotation(Annotation):
    def __init__(self, block_addr, stmt_idx, ins_addr):
        super(VariableSourceAnnotation, self).__init__()

        self.block_addr = block_addr
        self.stmt_idx = stmt_idx
        self.ins_addr = ins_addr

    @property
    def eliminatable(self):
        return False

    @property
    def relocatable(self):
        return False

    def __hash__(self):
        return hash((self.block_addr, self.stmt_idx, self.ins_addr))

    def __eq__(self, other):
        if not isinstance(other, VariableSourceAnnotation):
            return False

        return self.block_addr == other.block_addr and \
               self.stmt_idx == other.stmt_idx and \
               self.ins_addr == other.ins_addr

    @staticmethod
    def from_state(state):
        return VariableSourceAnnotation(state.scratch.bbl_addr, state.scratch.stmt_idx, state.scratch.ins_addr)



import angr


class GetProcessHeap(angr.SimProcedure):
    def run(self):
        return 1  # fake heap handle


class HeapCreate(angr.SimProcedure):
    def run(self, flOptions, dwInitialSize, dwMaximumSize): #pylint:disable=arguments-differ, unused-argument
        return 1  # still a fake heap handle


class HeapAlloc(angr.SimProcedure):
    def run(self, HeapHandle, Flags, Size): #pylint:disable=arguments-differ, unused-argument
        addr = self.state.heap._malloc(Size)

        # conditionally zero the allocated memory
        if self.state.solver.solution(Flags & 8, 8):
            if isinstance(self.state.heap, angr.SimHeapPTMalloc):
                # allocated size might be greater than requested
                data_size = self.state.solver.eval_one(
                    self.state.heap.chunk_from_mem(addr).get_data_size()
                )
            else:
                data_size = self.state.heap._conc_alloc_size(Size)
            data = self.state.solver.BVV(0, data_size * 8)
            self.state.memory.store(addr, data, size=data_size, condition=Flags & 8 == 8)
        return addr


class HeapReAlloc(angr.SimProcedure):
    def run(self, hHeap, dwFlags, lpMem, dwBytes): #pylint:disable=arguments-differ, unused-argument
        return self.state.heap._realloc(lpMem, dwBytes)


class GlobalAlloc(HeapAlloc):
    def run(self, Flags, Size):
        return super(GlobalAlloc, self).run(1, Flags, Size)


class HeapFree(angr.SimProcedure):
    def run(self, HeapHandle, Flags, lpMem): #pylint:disable=arguments-differ, unused-argument
        return 1



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("xinputuap.dll")
prototypes = \
    {
        # 
        'XInputGetState': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwPacketNumber": SimTypeInt(signed=False, label="UInt32"), "Gamepad": SimStruct({"wButtons": SimTypeShort(signed=False, label="UInt16"), "bLeftTrigger": SimTypeChar(label="Byte"), "bRightTrigger": SimTypeChar(label="Byte"), "sThumbLX": SimTypeShort(signed=True, label="Int16"), "sThumbLY": SimTypeShort(signed=True, label="Int16"), "sThumbRX": SimTypeShort(signed=True, label="Int16"), "sThumbRY": SimTypeShort(signed=True, label="Int16")}, name="XINPUT_GAMEPAD", pack=False, align=None)}, name="XINPUT_STATE", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwUserIndex", "pState"]),
        # 
        'XInputSetState': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"wLeftMotorSpeed": SimTypeShort(signed=False, label="UInt16"), "wRightMotorSpeed": SimTypeShort(signed=False, label="UInt16")}, name="XINPUT_VIBRATION", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwUserIndex", "pVibration"]),
        # 
        'XInputGetCapabilities': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Type": SimTypeChar(label="Byte"), "SubType": SimTypeChar(label="Byte"), "Flags": SimTypeShort(signed=False, label="UInt16"), "Gamepad": SimStruct({"wButtons": SimTypeShort(signed=False, label="UInt16"), "bLeftTrigger": SimTypeChar(label="Byte"), "bRightTrigger": SimTypeChar(label="Byte"), "sThumbLX": SimTypeShort(signed=True, label="Int16"), "sThumbLY": SimTypeShort(signed=True, label="Int16"), "sThumbRX": SimTypeShort(signed=True, label="Int16"), "sThumbRY": SimTypeShort(signed=True, label="Int16")}, name="XINPUT_GAMEPAD", pack=False, align=None), "Vibration": SimStruct({"wLeftMotorSpeed": SimTypeShort(signed=False, label="UInt16"), "wRightMotorSpeed": SimTypeShort(signed=False, label="UInt16")}, name="XINPUT_VIBRATION", pack=False, align=None)}, name="XINPUT_CAPABILITIES", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwUserIndex", "dwFlags", "pCapabilities"]),
        # 
        'XInputEnable': SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeBottom(label="Void"), arg_names=["enable"]),
        # 
        'XInputGetAudioDeviceIds': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwUserIndex", "pRenderDeviceId", "pRenderCount", "pCaptureDeviceId", "pCaptureCount"]),
        # 
        'XInputGetBatteryInformation': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeChar(label="Byte"), SimTypePointer(SimStruct({"BatteryType": SimTypeChar(label="Byte"), "BatteryLevel": SimTypeChar(label="Byte")}, name="XINPUT_BATTERY_INFORMATION", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwUserIndex", "devType", "pBatteryInformation"]),
        # 
        'XInputGetKeystroke': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"VirtualKey": SimTypeInt(signed=False, label="XINPUT_VIRTUAL_KEY"), "Unicode": SimTypeChar(label="Char"), "Flags": SimTypeShort(signed=False, label="UInt16"), "UserIndex": SimTypeChar(label="Byte"), "HidCode": SimTypeChar(label="Byte")}, name="XINPUT_KEYSTROKE", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwUserIndex", "dwReserved", "pKeystroke"]),
    }

lib.set_prototypes(prototypes)



from typing import Dict, Union
from itertools import count

from ... import sim_type
from ...sim_type import SimType
from . import typeconsts
from .typeconsts import TypeConstant


class SimTypeTempRef(sim_type.SimType):
    def __init__(self, typevar):
        super().__init__()
        self.typevar = typevar

    def c_repr(self):
        return "<SimTypeTempRef>"


class TypeTranslator:
    """
    Translate type variables to SimType equivalence.
    """
    def __init__(self, arch=None):

        self.arch = arch

        self.translated: Dict[TypeConstant,SimType] = { }
        self.translated_simtypes: Dict[SimType,TypeConstant] = { }
        self.structs = { }
        self._struct_ctr = count()

        # will be updated every time .translate() is called
        self._has_nonexistent_ref = False

    #
    # Naming
    #

    def struct_name(self):

        return "struct_%d" % next(self._struct_ctr)

    #
    # Type translation
    #

    def tc2simtype(self, tc):

        self._has_nonexistent_ref = False
        return self._tc2simtype(tc), self._has_nonexistent_ref

    def _tc2simtype(self, tc):

        if tc is None:
            return sim_type.SimTypeBottom().with_arch(self.arch)

        try:
            handler = TypeConstHandlers[tc.__class__]
        except KeyError:
            return sim_type.SimTypeBottom().with_arch(self.arch)

        translated = handler(self, tc)
        return translated

    def simtype2tc(self, simtype: sim_type.SimType) -> typeconsts.TypeConstant:
        return self._simtype2tc(simtype)

    def _simtype2tc(self, simtype: sim_type.SimType) -> typeconsts.TypeConstant:
        if simtype in self.translated_simtypes:
            return self.translated_simtypes[simtype]
        try:
            handler = SimTypeHandlers[simtype.__class__]
            return handler(self, simtype)
        except KeyError:
            return typeconsts.BottomType()

    #
    # Typehoon type handlers
    #

    def _translate_Pointer64(self, tc):

        if isinstance(tc.basetype, typeconsts.BottomType):
            # void *
            internal = sim_type.SimTypeBottom(label="void").with_arch(self.arch)
        else:
            internal = self._tc2simtype(tc.basetype)
        return sim_type.SimTypePointer(internal).with_arch(self.arch)

    def _translate_Pointer32(self, tc):

        if isinstance(tc.basetype, typeconsts.BottomType):
            # void *
            internal = sim_type.SimTypeBottom(label="void").with_arch(self.arch)
        else:
            internal = self._tc2simtype(tc.basetype)
        return sim_type.SimTypePointer(internal).with_arch(self.arch)

    def _translate_Array(self, tc: typeconsts.Array):
        elem_type = self._tc2simtype(tc.element)
        return sim_type.SimTypeArray(elem_type, length=tc.count).with_arch(self.arch)

    def _translate_Struct(self, tc):

        if tc in self.structs:
            return self.structs[tc]

        s = sim_type.SimStruct({}, name=self.struct_name()).with_arch(self.arch)
        self.structs[tc] = s

        next_offset = 0
        for offset, typ in sorted(tc.fields.items(), key=lambda item: item[0]):
            if offset > next_offset:
                # we need padding!
                padding_size = offset - next_offset
                s.fields["padding_%x" % next_offset] = sim_type.SimTypeFixedSizeArray(
                    sim_type.SimTypeChar(signed=False).with_arch(self.arch), padding_size
                ).with_arch(self.arch)

            translated_type = self._tc2simtype(typ)
            if isinstance(translated_type, sim_type.SimTypeBottom):
                # we cannot have bottom types in a struct since that would mess with offsets of all future types
                # for now, we replace it with an unsigned char
                translated_type = sim_type.SimTypeChar(signed=False).with_arch(self.arch)

            s.fields["field_%x" % offset] = translated_type

            if isinstance(translated_type, SimTypeTempRef):
                next_offset = self.arch.bytes + offset
            else:
                next_offset = translated_type.size // self.arch.byte_width + offset

        return s

    def _translate_Int8(self, tc):  # pylint:disable=unused-argument
        return sim_type.SimTypeChar(signed=False).with_arch(self.arch)

    def _translate_Int16(self, tc):  # pylint:disable=unused-argument
        return sim_type.SimTypeShort(signed=False).with_arch(self.arch)

    def _translate_Int32(self, tc):  # pylint:disable=unused-argument
        return sim_type.SimTypeInt(signed=False).with_arch(self.arch)

    def _translate_Int64(self, tc):  # pylint:disable=unused-argument
        return sim_type.SimTypeLongLong(signed=False).with_arch(self.arch)

    def _translate_Int128(self, tc):  # pylint:disable=unused-argument
        return sim_type.SimTypeNum(128, signed=False).with_arch(self.arch)

    def _translate_TypeVariableReference(self, tc):

        if tc.typevar in self.translated:
            return self.translated[tc.typevar]

        self._has_nonexistent_ref = True
        return SimTypeTempRef(tc.typevar)

    #
    # Backpatching
    #

    def backpatch(self, st, translated):
        """

        :param sim_type.SimType st:
        :param dict translated:
        :return:
        """

        if isinstance(st, sim_type.SimTypePointer):
            self.backpatch(st.pts_to, translated)

        elif isinstance(st, sim_type.SimStruct):
            fields_patch = { }
            for offset, fld in st.fields.items():
                if isinstance(fld, SimTypeTempRef) and fld.typevar in translated:
                    fields_patch[offset] = translated[fld.typevar]
                st.fields.update(fields_patch)

    #
    # SimType handlers
    #

    def _translate_SimTypeInt(self, st: sim_type.SimTypeInt) -> typeconsts.Int32:
        return typeconsts.Int32()

    def _translate_SimTypeLong(self, st: sim_type.SimTypeLong) -> typeconsts.Int32:
        return typeconsts.Int32()

    def _translate_SimTypeLongLong(self, st: sim_type.SimTypeLongLong) -> typeconsts.Int64:
        return typeconsts.Int64()

    def _translate_SimTypeChar(self, st: sim_type.SimTypeChar) -> typeconsts.Char:
        return typeconsts.Char()

    def _translate_SimStruct(self, st: sim_type.SimStruct) -> typeconsts.Struct:
        fields = { }
        offsets = st.offsets
        for name, ty in st.fields.items():
            offset = offsets[name]
            fields[offset] = self._simtype2tc(ty)

        return typeconsts.Struct(fields=fields)

    def _translate_SimTypeArray(self, st: sim_type.SimTypeArray) -> typeconsts.Array:
        elem_type = self._simtype2tc(st.elem_type)
        array_tc = typeconsts.Array(elem_type, count=st.length)
        return array_tc

    def _translate_SimTypePointer(self,
                                  st: sim_type.SimTypePointer) -> Union[typeconsts.Pointer32, typeconsts.Pointer64]:
        base = self._simtype2tc(st.pts_to)
        if self.arch.bits == 32:
            return typeconsts.Pointer32(base)
        elif self.arch.bits == 64:
            return typeconsts.Pointer64(base)
        raise TypeError("Unsupported pointer size %d" % self.arch.bits)


TypeConstHandlers = {
    typeconsts.Pointer64: TypeTranslator._translate_Pointer64,
    typeconsts.Pointer32: TypeTranslator._translate_Pointer32,
    typeconsts.Array: TypeTranslator._translate_Array,
    typeconsts.Struct: TypeTranslator._translate_Struct,
    typeconsts.Int8: TypeTranslator._translate_Int8,
    typeconsts.Int16: TypeTranslator._translate_Int16,
    typeconsts.Int32: TypeTranslator._translate_Int32,
    typeconsts.Int64: TypeTranslator._translate_Int64,
    typeconsts.Int128: TypeTranslator._translate_Int128,
    typeconsts.TypeVariableReference: TypeTranslator._translate_TypeVariableReference,
}


SimTypeHandlers = {
    sim_type.SimTypePointer: TypeTranslator._translate_SimTypePointer,
    sim_type.SimTypeInt: TypeTranslator._translate_SimTypeInt,
    sim_type.SimTypeLong: TypeTranslator._translate_SimTypeLong,
    sim_type.SimTypeLongLong: TypeTranslator._translate_SimTypeLongLong,
    sim_type.SimTypeChar: TypeTranslator._translate_SimTypeChar,
    sim_type.SimStruct: TypeTranslator._translate_SimStruct,
    sim_type.SimTypeArray: TypeTranslator._translate_SimTypeArray,
}




import logging

from .base import SimSootExpr

l = logging.getLogger('angr.engines.soot.expressions.phi')


class SimSootExpr_Phi(SimSootExpr):
    def _execute(self):
        # One value will be defined, based on the taken path
        # We first try to take the value from the source block, otherwise we iterate to find a define value
        # One case in which the value is NOT in the source block is when the source block is in native code
        if hasattr(self.state.scratch.source, 'block_idx'):
            local_values = [self._translate_value(v) for v, idx in self.expr.values if idx == self.state.scratch.source.block_idx]
            if len(local_values) > 0:
                # fastpath
                local = local_values[0]
                value = self.state.memory.load(local, none_if_missing=True)
                self.expr = value
                return

        local_options = [self._translate_value(v) for v, idx in self.expr.values[::-1]]
        for local in local_options:
            value = self.state.memory.load(local, none_if_missing=True)
            if value is not None:
                self.expr = value
                return



import logging

from angr.procedures.stubs.format_parser import FormatParser

l = logging.getLogger(name=__name__)

class printf(FormatParser):
    def run(self):
        stdout = self.state.posix.get_fd(1)
        if stdout is None:
            return -1

        # The format str is at index 0
        fmt_str = self._parse(0)
        out_str = fmt_str.replace(1, self.arg)

        stdout.write_data(out_str, out_str.size() // 8)
        return out_str.size() // 8

class __printf_chk(FormatParser):
    def run(self):
        stdout = self.state.posix.get_fd(1)
        if stdout is None:
            return -1

        # The format str is at index 1
        fmt_str = self._parse(1)
        out_str = fmt_str.replace(2, self.arg)

        stdout.write_data(out_str, out_str.size() // 8)
        return out_str.size() // 8



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_8048_LE_16_default(ArchPcode):
    name = '8048:LE:16:default'
    pcode_arch = '8048:LE:16:default'
    description = '8048 Microcontroller Family'
    bits = 16
    ip_offset = 0x20
    sp_offset = 0x1
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('a', 1, 0x0),
        Register('sp', 1, 0x1),
        Register('r0', 1, 0x10),
        Register('r1', 1, 0x11),
        Register('r2', 1, 0x12),
        Register('r3', 1, 0x13),
        Register('r4', 1, 0x14),
        Register('r5', 1, 0x15),
        Register('r6', 1, 0x16),
        Register('r7', 1, 0x17),
        Register('pc', 2, 0x20, alias_names=('ip',)),
        Register('c', 1, 0x30),
        Register('ac', 1, 0x31),
        Register('f0', 1, 0x32),
        Register('f1', 1, 0x33),
        Register('bs', 1, 0x34),
        Register('dfb', 1, 0x35)
    ]

register_arch(['8048:le:16:default'], 16, Endness.LE, ArchPcode_8048_LE_16_default)



import angr

######################################
# setsockopt
######################################

class setsockopt(angr.SimProcedure):
    #pylint:disable=arguments-differ

    def run(self, sockfd, level, optname, optval, optmain):
        return 0



import claripy

from .sim_type import SimTypePointer as Ptr, SimTypeTop as Top

class TypedValue(claripy.BackendObject):
    def __init__(self, ty, value):
        self.ty = ty
        self.value = value

    def __repr__(self):
        return 'TypedValue(%s, %s)' % (repr(self.ty), repr(self.value))

class TypeBackend(claripy.Backend):
    def __init__(self):
        super(TypeBackend, self).__init__(solver_required=False)

        self._op_expr['BVS'] = self._make_top
        self._op_expr['BVV'] = self._make_top

        self._op_raw['__add__'] = self._do_add
        self._op_raw['__sub__'] = self._do_sub
        self._op_raw['__and__'] = self._do_and
        self._op_raw['__or__'] = self._do_or
        self._op_raw['__xor__'] = self._do_xor

    @staticmethod
    def _make_top(ast, **kwargs):   # pylint: disable=unused-argument
        return TypedValue(Top(label=[]), ast)

    def _do_add(self, *args):
        if len(args) != 2:
            return reduce(self._do_add, args)

        a, b = args
        good_a = type(a.ty) is Ptr
        good_b = type(b.ty) is Ptr
        val = a.value + b.value
        out = TypedValue(Top(), val)

        if good_a:
            if not good_b:
                out.ty = Ptr(a.ty.pts_to, offset=a.ty.offset + b.value)
        elif good_b:
            out.ty = Ptr(b.ty.pts_to, offset=b.ty.offset + a.value)
        else:
            out.ty = Top()

        out.ty.label = a.ty.label + b.ty.label
        return out

    def _do_and(self, *args):
        if len(args) != 2:
            return reduce(self._do_and, args)

        a, b = args
        good_a = type(a.ty) is Ptr
        good_b = type(b.ty) is Ptr
        val = a.value & b.value
        out = TypedValue(Top(), val)

        if good_a:
            if not good_b:
                out.ty = Ptr(a.ty.pts_to, offset=a.ty.offset & b.value)
        elif good_b:
            out.ty = Ptr(b.ty.pts_to, offset=b.ty.offset & a.value)
        else:
            out.ty = Top()

        out.ty.label = a.ty.label + b.ty.label
        return out

    def _do_or(self, *args):
        if len(args) != 2:
            return reduce(self._do_or, args)

        a, b = args
        good_a = type(a.ty) is Ptr
        good_b = type(b.ty) is Ptr
        val = a.value | b.value
        out = TypedValue(Top(), val)

        if good_a:
            if not good_b:
                out.ty = Ptr(a.ty.pts_to, offset=a.ty.offset | b.value)
        elif good_b:
            out.ty = Ptr(b.ty.pts_to, offset=b.ty.offset | a.value)
        else:
            out.ty = Top(label=[])

        out.ty.label = a.ty.label + b.ty.label
        return out

    def _do_xor(self, *args):
        if len(args) != 2:
            return reduce(self._do_xor, args)

        a, b = args
        good_a = type(a.ty) is Ptr
        good_b = type(b.ty) is Ptr
        val = a.value ^ b.value
        out = TypedValue(Top(), val)

        if good_a:
            if not good_b:
                out.ty = Ptr(a.ty.pts_to, offset=a.ty.offset ^ b.value)
        elif good_b:
            out.ty = Ptr(b.ty.pts_to, offset=b.ty.offset ^ a.value)
        else:
            out.ty = Top()

        out.ty.label = a.ty.label + b.ty.label
        return out

    def _do_sub(self, *args):
        if len(args) != 2:
            return reduce(self._do_sub, args)

        a, b = args
        good_a = type(a.ty) is Ptr
        good_b = type(b.ty) is Ptr
        val = a.value - b.value
        out = TypedValue(Top(None), val)

        if good_a and not good_b:
            out.ty = Ptr(a.ty.pts_to, offset=a.ty.offset - b.value)
        else:
            out.ty = Top()

        out.ty.label = a.ty.label + b.ty.label
        return out

    def apply_annotation(self, obj, a):
        if type(a) is TypeAnnotation:
            return TypedValue(a.ty, obj.value)
        return obj

    @staticmethod
    def default_op(expr):
        return TypedValue(Top(label=[]), expr)

class TypeAnnotation(claripy.Annotation):
    def __init__(self, ty):
        self.ty = ty

    @property
    def eliminatable(self): #pylint:disable=no-self-use
        return False

    @property
    def relocatable(self): #pylint:disable=no-self-use
        return False
